/*============================================================================
  File: ExtendedEvents_Azure.sql

  SQL Server Versions:  Azure SQL Database
------------------------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com
  
  (c) 2020, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

/*
	Azure SQL DB
*/
SELECT @@VERSION;
GO

SELECT name, description
FROM sys.dm_xe_objects
WHERE object_type = 'event'
ORDER BY name;
GO

SELECT name, description
FROM sys.dm_xe_objects
WHERE object_type = 'pred_source'
ORDER BY name;
GO

SELECT name, description
FROM sys.dm_xe_objects
WHERE object_type = 'action'
ORDER BY name;
GO

SELECT name, description
FROM sys.dm_xe_objects
WHERE object_type = 'target'
ORDER BY name;
GO


IF NOT EXISTS
    (SELECT * FROM sys.symmetric_keys
        WHERE symmetric_key_id = 101)
BEGIN
    CREATE MASTER KEY ENCRYPTION BY PASSWORD = '0C34C960-6621-4682-A123-C7EA08E3FC46' -- Or any newid().
END
GO

/* 
	Need Azure StorageAccount name, and the associated Container name
*/
IF EXISTS
    (SELECT * FROM sys.database_scoped_credentials
        WHERE name = 'https://extendedeventsfile.blob.core.windows.net/demos')
BEGIN
    DROP DATABASE SCOPED CREDENTIAL
       [https://xeeventfiletarget.blob.core.windows.net/demo] ;
END
GO


/* 
	Need Azure StorageAccount name, and the associated Container name
	use '.blob.',   and not '.queue.' or '.table.' etc.

	For the SEcRET: Paste in the long SasToken string, but exclude any leading '?'
*/
CREATE
    DATABASE SCOPED
    CREDENTIAL
        [https://xeeventfiletarget.blob.core.windows.net/demo]
    WITH
        IDENTITY = 'SHARED ACCESS SIGNATURE',  -- "SAS" token.
        SECRET = 'sv=2019-02-02&ss=btqf&srt=sco&st=2020-06-26T16%3A41%3A02Z&se=2020-06-30T16%3A41%3A00Z&sp=rwdlacup&sig=uwtLB%2BHbAC5NFsRH4iS5pWDS4acGjTJTEdhUPML4KAk%3D'
    ;
GO



CREATE EVENT SESSION [XEAzure_Queries] 
	ON DATABASE 
ADD EVENT sqlserver.sp_statement_completed,
ADD EVENT sqlserver.sql_statement_completed
ADD TARGET package0.event_file(
	SET filename=N'https://xeeventfiletarget.blob.core.windows.net/demo/xeazurequeries.xel',
	max_file_size=(128)
	)
WITH (MAX_MEMORY=4096 KB,EVENT_RETENTION_MODE=ALLOW_SINGLE_EVENT_LOSS,MAX_DISPATCH_LATENCY=30 SECONDS,
MAX_EVENT_SIZE=0 KB,MEMORY_PARTITION_MODE=NONE,TRACK_CAUSALITY=OFF,STARTUP_STATE=OFF);
GO


ALTER EVENT SESSION [XEAzure_Queries]
	ON DATABASE
	STATE = START;
GO


SELECT *
FROM Application.People;
GO

SELECT TOP 100 *
FROM Sales.Orders;
GO


ALTER EVENT SESSION [XEAzure_Queries]
	ON DATABASE
	STATE = STOP;
GO


DROP EVENT SESSION [XEAzure_Queries] 
	ON DATABASE;
GO