/*============================================================================
  File:     6a_SystemHealth.sql

  SQL Server Versions: 2012 onwards
------------------------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com
  
  (c) 2018, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

/*
	Event session definition for the system_health session
	If you happen to delete it or modify it and need to re-set:
		C:\Program Files\Microsoft SQL Server\MSSQL12.SQL2014\MSSQL\Install\u_tables.sql

	Note: The predicates in this session are designed to minimize impact of event collection.
	Changing the predicate definition may impact system performance.
*/

CREATE EVENT SESSION [system_health] ON SERVER 
ADD EVENT sqlclr.clr_allocation_failure(
    ACTION(package0.callstack,sqlserver.session_id)),
ADD EVENT sqlclr.clr_virtual_alloc_failure(
    ACTION(package0.callstack,sqlserver.session_id)),
ADD EVENT sqlos.memory_broker_ring_buffer_recorded,
ADD EVENT sqlos.memory_node_oom_ring_buffer_recorded(
    ACTION(package0.callstack,sqlserver.session_id,sqlserver.sql_text,sqlserver.tsql_stack)),
ADD EVENT sqlos.scheduler_monitor_deadlock_ring_buffer_recorded,
/*
	Capture any non-yielding scheduler problems (These appear in the SQL Server error log as error 17883.)
*/
ADD EVENT sqlos.scheduler_monitor_non_yielding_iocp_ring_buffer_recorded,
ADD EVENT sqlos.scheduler_monitor_non_yielding_ring_buffer_recorded,
ADD EVENT sqlos.scheduler_monitor_non_yielding_rm_ring_buffer_recorded,
ADD EVENT sqlos.scheduler_monitor_stalled_dispatcher_ring_buffer_recorded,
ADD EVENT sqlos.scheduler_monitor_system_health_ring_buffer_recorded,
ADD EVENT sqlos.wait_info(
    ACTION(package0.callstack,sqlserver.session_id,sqlserver.sql_text)
    WHERE 
		/*
			The callstack, sql_text, and session_id for any sessions that have waited on latches (or other interesting resources) for > 15 seconds.
		*/
		(
			[duration]>(15000) AND (
				[wait_type]>=N'LATCH_NL' AND (
					[wait_type]>=N'PAGELATCH_NL' AND [wait_type]<=N'PAGELATCH_DT' OR [wait_type]<=N'LATCH_DT' 
					OR [wait_type]>=N'PAGEIOLATCH_NL' AND [wait_type]<=N'PAGEIOLATCH_DT' OR [wait_type]>=N'IO_COMPLETION' 
					AND [wait_type]<=N'NETWORK_IO' OR [wait_type]=N'RESOURCE_SEMAPHORE' 
					OR [wait_type]=N'SOS_WORKER' OR [wait_type]>=N'FCB_REPLICA_WRITE' 
					AND [wait_type]<=N'WRITELOG' OR [wait_type]=N'CMEMTHREAD' OR [wait_type]=N'TRACEWRITE' 
					OR [wait_type]=N'RESOURCE_SEMAPHORE_MUTEX'
					) 
				/*
					Waits for locks that have exceeded 30 secs.
				*/
				OR [duration]>(30000) AND [wait_type]<=N'LCK_M_RX_X')
			)
		),
ADD EVENT sqlos.wait_info_external(
    ACTION(package0.callstack,sqlserver.session_id,sqlserver.sql_text)
    WHERE 
		/*
			Login related preemptive waits that have exceeded 5 seconds.
		*/
		(
			[duration]>(5000) AND (
				[wait_type]>=N'PREEMPTIVE_OS_GENERICOPS' AND [wait_type]<=N'PREEMPTIVE_OS_ENCRYPTMESSAGE' 
				OR	[wait_type]>=N'PREEMPTIVE_OS_INITIALIZESECURITYCONTEXT' AND [wait_type]<=N'PREEMPTIVE_OS_QUERYSECURITYCONTEXTTOKEN' 
				OR	[wait_type]>=N'PREEMPTIVE_OS_AUTHZGETINFORMATIONFROMCONTEXT' AND [wait_type]<=N'PREEMPTIVE_OS_REVERTTOSELF' 
				OR	[wait_type]>=N'PREEMPTIVE_OS_CRYPTACQUIRECONTEXT' AND [wait_type]<=N'PREEMPTIVE_OS_DEVICEOPS' 
				OR	[wait_type]>=N'PREEMPTIVE_OS_NETGROUPGETUSERS' AND [wait_type]<=N'PREEMPTIVE_OS_NETUSERMODALSGET' 
				OR	[wait_type]>=N'PREEMPTIVE_OS_NETVALIDATEPASSWORDPOLICYFREE' AND [wait_type]<=N'PREEMPTIVE_OS_DOMAINSERVICESOPS' 
				OR	[wait_type]=N'PREEMPTIVE_OS_VERIFYSIGNATURE' 
				OR	
				[duration]>(45000) AND (
					[wait_type]>=N'PREEMPTIVE_OS_SETNAMEDSECURITYINFO' AND [wait_type]<=N'PREEMPTIVE_CLUSAPI_CLUSTERRESOURCECONTROL' 
				OR	[wait_type]>=N'PREEMPTIVE_OS_RSFXDEVICEOPS' AND [wait_type]<=N'PREEMPTIVE_OS_DSGETDCNAME' 
				OR	[wait_type]>=N'PREEMPTIVE_OS_DTCOPS' AND [wait_type]<=N'PREEMPTIVE_DTC_ABORT' 
				OR	[wait_type]>=N'PREEMPTIVE_OS_CLOSEHANDLE' AND [wait_type]<=N'PREEMPTIVE_OS_FINDFILE' 
				OR	[wait_type]>=N'PREEMPTIVE_OS_GETCOMPRESSEDFILESIZE' AND [wait_type]<=N'PREEMPTIVE_ODBCOPS' 
				OR	[wait_type]>=N'PREEMPTIVE_OS_DISCONNECTNAMEDPIPE' AND [wait_type]<=N'PREEMPTIVE_CLOSEBACKUPMEDIA' 
				OR	[wait_type]=N'PREEMPTIVE_OS_AUTHENTICATIONOPS' OR [wait_type]=N'PREEMPTIVE_OS_FREECREDENTIALSHANDLE' 
				OR	[wait_type]=N'PREEMPTIVE_OS_AUTHORIZATIONOPS' OR [wait_type]=N'PREEMPTIVE_COM_COCREATEINSTANCE' 
				OR	[wait_type]=N'PREEMPTIVE_OS_NETVALIDATEPASSWORDPOLICY' OR [wait_type]=N'PREEMPTIVE_VSS_CREATESNAPSHOT'
				)
			)
		)
	),
ADD EVENT sqlserver.connectivity_ring_buffer_recorded(SET collect_call_stack=(1)),
/*
	Get callstack, SPID, and query for all high severity errors ( above severity 20 ) and all OOM errors ( 17803 , 701 , 802 , 8645 , 8651 , 8657 , 8902 )
*/
ADD EVENT sqlserver.error_reported(
    ACTION(package0.callstack,sqlserver.database_id,sqlserver.session_id,sqlserver.sql_text,sqlserver.tsql_stack)
    WHERE (
		[severity]>=(20) 
		OR (
			[error_number]=(17803) OR [error_number]=(701) OR [error_number]=(802) 
			OR [error_number]=(8645) OR [error_number]=(8651) OR [error_number]=(8657) 
			OR [error_number]=(8902) OR [error_number]=(41354) OR [error_number]=(41355) 
			OR [error_number]=(41367) OR [error_number]=(41384) OR [error_number]=(41336) 
			OR [error_number]=(41309) OR [error_number]=(41312) OR [error_number]=(41313)
			)
		)
	),
ADD EVENT sqlserver.security_error_ring_buffer_recorded(SET collect_call_stack=(1)),
ADD EVENT sqlserver.sp_server_diagnostics_component_result(SET collect_data=(1)
    WHERE ([sqlserver].[is_system]=(1) AND [component]<>(4))),
/*	
	Deadlocks!
*/
ADD EVENT sqlserver.xml_deadlock_report 
/*
	write to a file (only in 2012 and higher)
*/
ADD TARGET package0.event_file(SET filename=N'system_health.xel',max_file_size=(5),max_rollover_files=(4)),
/*
	write to the ring_buffer...you may not see everything here due to the limitation in the DMV
*/
ADD TARGET package0.ring_buffer(SET max_events_limit=(5000),max_memory=(4096))
WITH (MAX_MEMORY=4096 KB,EVENT_RETENTION_MODE=ALLOW_SINGLE_EVENT_LOSS,MAX_DISPATCH_LATENCY=120 SECONDS,MAX_EVENT_SIZE=0 KB,MEMORY_PARTITION_MODE=NONE,TRACK_CAUSALITY=OFF,STARTUP_STATE=ON)
GO




/*
	look at the ring buffer output
*/
SELECT CAST(st.target_data AS XML) 
FROM sys.dm_xe_session_targets st
JOIN sys.dm_xe_sessions s ON (s.address = st.event_session_address)
WHERE s.name = 'system_health' 
	AND st.target_name = N'ring_buffer';


/*
	push into an XML variable, then read out information
	(but it's probably easier to look at the file output)
*/
DECLARE @target_data XML;
SELECT @target_data = CAST(target_data AS XML)
FROM sys.dm_xe_sessions AS s 
JOIN sys.dm_xe_session_targets AS t 
    ON t.event_session_address = s.address
WHERE s.name = N'system_health'
  AND t.target_name = N'ring_buffer';

SELECT 
    @target_data.value('(RingBufferTarget/@eventsPerSec)[1]', 'int') AS eventsPerSec,
    @target_data.value('(RingBufferTarget/@processingTime)[1]', 'int') AS processingTime,
    @target_data.value('(RingBufferTarget/@totalEventsProcessed)[1]', 'int') AS totalEventsProcessed,
    @target_data.value('(RingBufferTarget/@eventCount)[1]', 'int') AS eventCount,
    @target_data.value('(RingBufferTarget/@droppedCount)[1]', 'int') AS droppedCount,
    @target_data.value('(RingBufferTarget/@memoryUsed)[1]', 'int') AS memoryUsed;

SELECT
    n.query('.') AS event_data
FROM @target_data.nodes('RingBufferTarget/event') AS q(n);

SELECT 
	q.n.value('@name', 'varchar(50)') AS EventName,
	q.n.value('@timestamp', 'datetime') AS DateAndTime,
	q.n.value('(data[@name="error"]/value)[1]', 'int') AS Error,
	q.n.value('(data[@name="process_utilization"]/value)[1]', 'int') AS CPU
FROM @target_data.nodes('RingBufferTarget/event') AS q(n);


/*
	Find errors
*/
DECLARE @errors XML;
SELECT @errors = CAST(target_data AS XML)
FROM sys.dm_xe_sessions AS s 
JOIN sys.dm_xe_session_targets AS t 
    ON t.event_session_address = s.address
WHERE s.name = N'system_health'
  AND t.target_name = N'ring_buffer';

SELECT
	DATEADD(mi, DATEDIFF(mi, GETUTCDATE(), CURRENT_TIMESTAMP),
	q.n.value('(@timestamp)[1]', 'datetime2')) AS [TimeStamp],
	q.n.value('(data[@name="severity"]/value)[1]', 'bigint') AS [Severity],
	q.n.value('(data[@name="error_number"]/value)[1]', 'bigint') AS [ErrorNumber],
	q.n.value('(data[@name="message"]/value)[1]', 'nvarchar(512)') AS [ErrorMessage],
	q.n.value('(action[@name="callstack"]/value)[1]', 'nvarchar(max)') AS [ErrorCallstack],
	q.n.query('.') as [event details]
FROM @errors.nodes('RingBufferTarget/event') AS q(n)
WHERE q.n.value('(@name)[1]', 'nvarchar(256)')='error_reported'
ORDER BY q.n.value('(@timestamp)[1]', 'datetime2');

/*
	What if we're interested in ALL previous errors
	(not just those in the ring_buffer)?
*/
SELECT
    object_name,
    event_data,
    file_name,
    file_offset
FROM sys.fn_xe_file_target_read_file(
	'C:\Program Files\Microsoft SQL Server\MSSQL13.BARTON\MSSQL\Log\system*', NULL, NULL, NULL
	)


SELECT 
	DATEADD(mi, DATEDIFF(mi, GETUTCDATE(), CURRENT_TIMESTAMP),
	q.n.value('(@timestamp)[1]', 'datetime2')) AS [TimeStamp],
	q.n.value('(data[@name="severity"]/value)[1]', 'bigint') AS [Severity],
	q.n.value('(data[@name="error_number"]/value)[1]', 'bigint') AS [ErrorNumber],
	q.n.value('(data[@name="message"]/value)[1]', 'nvarchar(512)') AS [ErrorMessage],
	q.n.value('(action[@name="callstack"]/value)[1]', 'nvarchar(max)') AS [ErrorCallstack],
	q.n.query('.') as [event details]
FROM 
(SELECT
    CAST(event_data AS XML) AS event_data
 FROM sys.fn_xe_file_target_read_file(
	'C:\Program Files\Microsoft SQL Server\MSSQL13.BARTON\MSSQL\Log\system*', NULL, NULL, NULL
	)
) as tab
CROSS APPLY event_data.nodes('event') as q(n)
WHERE q.n.value('(@name)[1]', 'nvarchar(256)')='error_reported'




/*
	Reporting Dashboard
	https://blogs.msdn.microsoft.com/sql_pfe_blog/2013/04/15/sql-2012-system-health-reporting-dashboard-visualizing-sp_server_diagnostics-results/
*/