/*============================================================================
  File:     5_Test_InMemTable.sql

  SQL Server Versions: 2016 onwards
------------------------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com
  
  (c) 2017, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

/*
	Test performance of in-memory table
*/
USE [InMemTesting];
GO

SET NOCOUNT ON;

DECLARE @NumRowsToInsert INT = 100000;
DECLARE @TestName VARCHAR(200) = 'InMemTable Insert - ' + CAST(@NumRowsToInsert AS VARCHAR(10)) + ' rows';

DECLARE @TestID INT;
DECLARE @DBID INT;

/* 
	Setup test info
*/

INSERT INTO [TestTracking].[dbo].[TrackTests] ([TestName]) VALUES (@TestName);
SELECT @TestID = MAX(TestID) FROM [TestTracking].[dbo].[TrackTests];
SELECT @DBID = DB_ID();

DROP TABLE IF EXISTS [#SQLskillsFileStats1];

DROP TABLE IF EXISTS [#SQLskillsFileStats2];
 
DROP TABLE IF EXISTS [#SQLskillsWaitStats1];

DROP TABLE IF EXISTS [#SQLskillsWaitStats2];

DROP TABLE IF EXISTS [#SQLskillsPerfCounters1];

DROP TABLE IF EXISTS [#SQLskillsPerfCounters2];

/*
	snapshot file stats, wait stats, perf counters
*/
SELECT [database_id], [file_id], [sample_ms], [num_of_reads], [num_of_bytes_read], [io_stall_read_ms],
[num_of_writes], [num_of_bytes_written], [io_stall_write_ms], [io_stall]
INTO [#SQLskillsFileStats1]
FROM sys.dm_io_virtual_file_stats(@DBID, NULL)

SELECT [wait_type], [waiting_tasks_count], [wait_time_ms],
  [max_wait_time_ms], [signal_wait_time_ms]
INTO [#SQLskillsWaitStats1]
FROM sys.dm_os_wait_stats;

SELECT SYSDATETIME() [SnapTime], [object_name], [counter_name], [instance_name],
[cntr_value], [cntr_type]
INTO [#SQLskillsPerfCounters1]
FROM [sys].[dm_os_performance_counters]
WHERE ([counter_name] = 'XTP Memory Used (KB)'
	AND ([instance_name] = '_Total' OR [instance_name] = 'InMem_OLTP_Test' OR [instance_name] = 'DiskTables_Test'))
OR ([counter_name] = 'XTP Controller Log Processed/sec'
	AND ([instance_name] = '_Total' OR [instance_name] = 'InMem_OLTP_Test' OR [instance_name] = 'DiskTables_Test'))
OR ([counter_name] = 'Lock Requests/sec' AND [instance_name] = '_Total')
OR ([counter_name] = 'Lock Waits/sec' AND [instance_name] = '_Total')
OR [counter_name] = 'Latch Waits/sec'
OR [counter_name] = 'Log Pool Memory (KB)';

/* 
	set start time 
*/
UPDATE [TestTracking].[dbo].[TrackTests] 
SET [TestStartTime] = SYSDATETIME() 
WHERE [TestID] = @TestID;

/*
	run inserts
*/
EXEC [InMemTesting].[dbo].[StandardSP_Inserts] @NumRowsToInsert;


/* 
	set the stop time 
*/
UPDATE [TestTracking].[dbo].[TrackTests] SET [TestEndTime] = SYSDATETIME() WHERE [TestID] = @TestID;

/*
	snapshot file stats, wait stats, perf counters again
*/
SELECT SYSDATETIME() [SnapTime], [object_name], [counter_name], [instance_name],
[cntr_value], [cntr_type]
INTO [#SQLskillsPerfCounters2]
FROM [sys].[dm_os_performance_counters]
WHERE ([counter_name] = 'XTP Memory Used (KB)'
	AND ([instance_name] = '_Total' OR [instance_name] = 'InMem_OLTP_Test' OR [instance_name] = 'DiskTables_Test'))
OR ([counter_name] = 'XTP Controller Log Processed/sec'
	AND ([instance_name] = '_Total' OR [instance_name] = 'InMem_OLTP_Test' OR [instance_name] = 'DiskTables_Test'))
OR ([counter_name] = 'Lock Requests/sec' AND [instance_name] = '_Total')
OR ([counter_name] = 'Lock Waits/sec' AND [instance_name] = '_Total')
OR [counter_name] = 'Latch Waits/sec'
OR [counter_name] = 'Log Pool Memory (KB)';

SELECT [wait_type], [waiting_tasks_count], [wait_time_ms],
  [max_wait_time_ms], [signal_wait_time_ms]
INTO #SQLskillsWaitStats2
FROM sys.dm_os_wait_stats;

SELECT [database_id], [file_id], [sample_ms], [num_of_reads], [num_of_bytes_read], [io_stall_read_ms],
[num_of_writes], [num_of_bytes_written], [io_stall_write_ms], [io_stall]
INTO #SQLskillsFileStats2
FROM sys.dm_io_virtual_file_stats(@DBID, NULL);

/*
	store perf counters
*/
INSERT INTO [TestTracking].[dbo].[PerfMonCounters] (
	[TestID], [ObjectName], [CounterName], [InstanceName], 
	[CounterValueDiff], [TimeDiff_ms], [CounterValue], [CounterUnits])
SELECT @TestID, [i].[object_name], [i].[counter_name], [i].[instance_name], 
CAST(([s].[cntr_value] - [i].[cntr_value]) AS DECIMAL (16,2)), 
DATEDIFF(MS, [i].[SnapTime], [s].[SnapTime]),
CASE
	WHEN [i].[cntr_type] = 65792 
		THEN [s].[cntr_value] - [i].[cntr_value]
	WHEN [i].[cntr_type] = 272696576 
		THEN (CAST(([s].[cntr_value] - i.[cntr_value]) AS DECIMAL (16,2))) / DATEDIFF(MS, [i].[SnapTime], [s].[SnapTime])
END,
CASE 
	WHEN [i].[cntr_type] = 65792 THEN 'Units'
	WHEN [i].[cntr_type] = 272696576 THEN 'per ms'
END 
FROM [#SQLskillsPerfCounters1] [i]
JOIN [#SQLskillsPerfCounters2] [s] 
	ON [i].[object_name] = [s].[object_name]
	AND [i].[counter_name] = [s].[counter_name]
	AND [i].[instance_name] = [s].[instance_name];

/* 
	store wait stats 
*/
WITH [DiffWaits] AS
(
  SELECT   -- Waits that weren't in the first snapshot
      [ts2].[wait_type],
      [ts2].[wait_time_ms],
      [ts2].[signal_wait_time_ms],
      [ts2].[waiting_tasks_count]
    FROM [#SQLskillsWaitStats2] AS [ts2]
    LEFT OUTER JOIN [#SQLskillsWaitStats1] AS [ts1]
    ON [ts2].[wait_type] = [ts1].[wait_type]
    WHERE [ts1].[wait_type] IS NULL
    AND [ts2].[wait_time_ms] > 0
  UNION
  SELECT   -- Diff of waits in both snapshots
      [ts2].[wait_type],
      [ts2].[wait_time_ms] - [ts1].[wait_time_ms] AS [wait_time_ms],
      [ts2].[signal_wait_time_ms] - [ts1].[signal_wait_time_ms] AS [signal_wait_time_ms],
      [ts2].[waiting_tasks_count] - [ts1].[waiting_tasks_count] AS [waiting_tasks_count]
    FROM [#SQLskillsWaitStats2] AS [ts2]
    LEFT OUTER JOIN [#SQLskillsWaitStats1] AS [ts1]
    ON [ts2].[wait_type] = [ts1].[wait_type]
    WHERE [ts1].[wait_type] IS NOT NULL
    AND [ts2].[waiting_tasks_count] - [ts1].[waiting_tasks_count] > 0
    AND [ts2].[wait_time_ms] - [ts1].[wait_time_ms] > 0
),
[Waits] AS
(
  SELECT
    [wait_type],
    [wait_time_ms] / 1000.0 AS [WaitS],
    ([wait_time_ms] - [signal_wait_time_ms]) / 1000.0 AS [ResourceS],
    [signal_wait_time_ms] / 1000.0 AS [SignalS],
    [waiting_tasks_count] AS [WaitCount],
    100.0 * [wait_time_ms] / SUM ([wait_time_ms]) OVER() AS [Percentage],
    ROW_NUMBER() OVER(ORDER BY [wait_time_ms] DESC) AS [RowNum]
  FROM [DiffWaits]
  WHERE [wait_type] NOT IN (SELECT WaitType FROM [TestTracking].[dbo].[WaitsToIgnore])
)

INSERT INTO [TestTracking].[dbo].[WaitStats]
(
	[TestID],
  [WaitType] ,
  [Wait_S] ,
  [Resource_S] ,
  [Signal_S] ,
  [WaitCount] ,
  [Percentage] ,
  [AvgWait_S] ,
  [AvgRes_S] ,
  [AvgSig_S]
)
SELECT
	@TestID,
  [W1].[wait_type],
  CAST ([W1].[WaitS] AS DECIMAL (16, 2)) ,
  CAST ([W1].[ResourceS] AS DECIMAL (16, 2)) ,
  CAST ([W1].[SignalS] AS DECIMAL (16, 2)) ,
  [W1].[WaitCount] ,
  CAST ([W1].[Percentage] AS DECIMAL (5, 2)) ,
  CAST (([W1].[WaitS] / [W1].[WaitCount]) AS DECIMAL (16, 4)) ,
  CAST (([W1].[ResourceS] / [W1].[WaitCount]) AS DECIMAL (16, 4)) ,
  CAST (([W1].[SignalS] / [W1].[WaitCount]) AS DECIMAL (16, 4))
FROM [Waits] AS [W1]
INNER JOIN [Waits] AS [W2]
ON [W2].[RowNum] <= [W1].[RowNum]
GROUP BY [W1].[RowNum], [W1].[wait_type], [W1].[WaitS], [W1].[ResourceS], 
  [W1].[SignalS], [W1].[WaitCount], [W1].[Percentage]
HAVING SUM ([W2].[Percentage]) - [W1].[Percentage] < 95; -- percentage threshold


/* store file stats */

INSERT INTO [TestTracking].[dbo].[FileStats]
           ([TestID]
           ,[database_id]
           ,[file_id]
           ,[sample_ms]
           ,[num_of_reads]
           ,[num_of_bytes_read]
           ,[io_stall_read_ms]
           ,[num_of_writes]
           ,[num_of_bytes_written]
           ,[io_stall_write_ms]
           ,[io_stall])
SELECT
@TestID,
f.database_id, f.file_id, 
(f.sample_ms - i.sample_ms) AS sample_ms, 
(f.num_of_reads - i.num_of_reads) AS num_of_reads,
(f.num_of_bytes_read - i.num_of_bytes_read ) AS num_of_bytes_read,
(f.io_stall_read_ms - i.io_stall_read_ms ) AS io_stall_read_ms,
(f.num_of_writes - i.num_of_writes ) AS num_of_writes,
(f.num_of_bytes_written - i.num_of_bytes_written ) AS num_of_bytes_written,
(f.io_stall_write_ms - i.io_stall_write_ms ) AS io_stall_write_ms,
(f.io_stall - i.io_stall ) AS io_stall
FROM #SQLskillsFileStats1 i
JOIN #SQLskillsFileStats2 f on i.database_id = f.database_id and i.file_id = f.file_id;





