USE [master];
GO

DROP DATABASE IF EXISTS [InMem_ReplaceTempdb];
GO

/*
	Create database 
*/
CREATE DATABASE [InMem_ReplaceTempdb]
 CONTAINMENT = NONE
 ON  PRIMARY 
( NAME = N'InMem_ReplaceTempdb', FILENAME = N'C:\Databases\InMem_ReplaceTempdb\InMem_ReplaceTempdb.mdf' , SIZE = 512MB , FILEGROWTH = 512MB )
 LOG ON 
( NAME = N'InMem_ReplaceTempdb_log', FILENAME = N'C:\Databases\InMem_ReplaceTempdb\InMem_ReplaceTempdb_log.ldf' , SIZE = 1024MB , FILEGROWTH = 1024MB )
GO

ALTER DATABASE [InMem_ReplaceTempdb] ADD FILEGROUP [InMem_OLTP]  
    CONTAINS MEMORY_OPTIMIZED_DATA;  
  
ALTER DATABASE [InMem_ReplaceTempdb] ADD FILE  
    (NAME = [InMem_OLTP_F1], FILENAME= 'C:\Databases\InMem_ReplaceTempdb\InMem_ReplaceTempdb_F1')  
    TO FILEGROUP [InMem_OLTP];  
GO

ALTER DATABASE [InMem_ReplaceTempdb] SET RECOVERY FULL;
GO

--ALTER DATABASE [InMem_ReplaceTempdb] SET MEMORY_OPTIMIZED_ELEVATE_TO_SNAPSHOT=ON
--GO
/*
	
*/
USE [InMem_ReplaceTempdb];
GO

/*
	Create SP that uses traditional temp table
*/
DROP PROCEDURE IF EXISTS [AbuseTempdb];  
GO

CREATE PROCEDURE [AbuseTempdb]
AS
	CREATE TABLE #temp (Col1 INT, Col2 NVARCHAR(4000))
	DECLARE @Counter INT = 0
	WHILE @Counter < 100
		BEGIN
			INSERT INTO #temp VALUES (@Counter, 'Alpha')
			SET @Counter = @Counter + 1
		END
GO

/*
	Create InMem table
*/
DROP TABLE IF EXISTS [dbo].[InMemtemp];
GO

CREATE TABLE [dbo].[InMemtemp] 
	(Col1 INT, Col2 NVARCHAR(4000), SessionIDFilter SMALLINT NOT NULL DEFAULT (@@spid),
	INDEX NCI_SessionIDFilter NONCLUSTERED (SessionIDFilter),
	CONSTRAINT Check_SessionIDFilter CHECK (SessionIDFilter = @@spid),)
	WITH (MEMORY_OPTIMIZED = ON,DURABILITY = SCHEMA_ONLY);
GO

/*
	Create InMem SP
*/
DROP PROCEDURE IF EXISTS [BeNiceToTempdb];
GO

CREATE PROCEDURE [BeNiceToTempdb]
WITH NATIVE_COMPILATION, SCHEMABINDING
AS
BEGIN ATOMIC WITH
(TRANSACTION ISOLATION LEVEL = SNAPSHOT, LANGUAGE = N'English')

DECLARE @Counter INT = 0
WHILE @Counter < 100
	BEGIN
		INSERT INTO [dbo].[InMemtemp] (Col1, Col2) VALUES (@Counter, 'Alpha')
		SET @Counter = @Counter + 1
	END

	DELETE FROM [dbo].[InMemtemp] WHERE SessionIDFilter = @@spid
END
GO

USE [master];
GO
ALTER DATABASE [InMem_ReplaceTempdb] SET QUERY_STORE (
	OPERATION_MODE = READ_WRITE,
	CLEANUP_POLICY = (STALE_QUERY_THRESHOLD_DAYS = 30),  
	DATA_FLUSH_INTERVAL_SECONDS = 60,
	INTERVAL_LENGTH_MINUTES = 5, 
	MAX_STORAGE_SIZE_MB = 512,
	QUERY_CAPTURE_MODE = ALL,
	SIZE_BASED_CLEANUP_MODE = AUTO,
	MAX_PLANS_PER_QUERY = 200);

GO

USE [InMem_ReplaceTempdb];
GO
/*
	Must execute SP before can enable query exec stats
*/
EXEC [dbo].[BeNiceToTempdb];
GO

/*
	Enable query exec stats
*/
DECLARE @DB_ID INT = DB_ID()  
DECLARE @SP_ID INT = OBJECT_ID('dbo.BeNiceToTempdb');  
DECLARE @collection_enabled BIT;  

EXEC [sys].[sp_xtp_control_query_exec_stats] 
	@new_collection_value = 1,   
    @database_id = @DB_ID, 
	@xtp_object_id = @SP_ID;  

-- Check the state of the collection flag  
EXEC sp_xtp_control_query_exec_stats @database_id = @db_id,   
    @xtp_object_id = @SP_ID,   
    @old_collection_value= @collection_enabled output;  
SELECT @collection_enabled AS 'collection status';  


/*
	Run external scripts
*/
USE [InMem_ReplaceTempdb];
GO

SELECT
	[qst].[query_text_id],
	[qsq].[query_id], 
	[qsp].[plan_id],
	OBJECT_NAME([qsq].[object_id]),
	[qst].[query_sql_text],
	TRY_CONVERT(XML, [qsp].[query_plan]),
	[qsp].[query_plan],
	[rs].[count_executions],
	[rs].[avg_duration],
	([rs].[count_executions]*[rs].[avg_duration])/1000 [TotalDuration_ms],
	[rs].[avg_logical_io_reads],
	[rs].[avg_cpu_time],
	([rs].[count_executions]*[rs].[avg_cpu_time]) [TotalCPU]
FROM [sys].[query_store_query] [qsq] 
JOIN [sys].[query_store_query_text] [qst]
	ON [qsq].[query_text_id] = [qst].[query_text_id]
JOIN [sys].[query_store_plan] [qsp] 
	ON [qsq].[query_id] = [qsp].[query_id]
JOIN [sys].[query_store_runtime_stats] [rs] 
	ON [qsp].[plan_id] = [rs].[plan_id]
WHERE [qsq].[object_id] <> 0
ORDER BY [qst].[query_text_id] DESC;
GO
    