/*============================================================================
  File:     08_TempTableVar.sql

  SQL Server Versions: 2016 onwards
------------------------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com
  
  (c) 2020, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

/*
	Restore database
	*you may need to change the backup
	and restore locations
*/
USE [master];
GO
RESTORE DATABASE [WideWorldImporters] 
FROM  DISK = N'C:\Backups\WideWorldImportersEnlarged.bak' 
WITH  FILE = 1,  
MOVE N'WWI_Primary' 
	TO N'C:\Databases\WideWorldImporters\WideWorldImporters.mdf',  
MOVE N'WWI_UserData' 
	TO N'C:\Databases\WideWorldImporters\WideWorldImporters_UserData.ndf',  
MOVE N'WWI_Log' 
	TO N'C:\Databases\WideWorldImporters\WideWorldImporters.ldf',  
MOVE N'WWI_InMemory_Data_1' 
	TO N'C:\Databases\WideWorldImporters\WideWorldImporters_InMemory_Data_1',  
NOUNLOAD, 
REPLACE, 
STATS = 5;
GO



/*
	
*/
USE [WideWorldImporters];
GO

CREATE NONCLUSTERED INDEX IX_Orders_MemTest
	ON Sales.Orders(
		CustomerID, 
		CustomerPurchaseOrderNumber
		)

/*
	Create SP that uses traditional temp table
*/
DROP PROCEDURE IF EXISTS [AbuseTempdb];  
GO

CREATE PROCEDURE [AbuseTempdb] @CustomerID INT = NULL
AS
BEGIN
	SET NOCOUNT ON

	CREATE TABLE #temp (
		[RowNum] INT IDENTITY (1,1),
		[OrderID] INT, 
		[CustomerID] INT,
		[CustomerPONum] NVARCHAR(40)
		)
	
	IF @CustomerID IS NULL
	BEGIN
		INSERT INTO #temp (
			[OrderID],
			[CustomerID],
			[CustomerPONum])
		SELECT
			[OrderID],
			[CustomerID],
			[CustomerPurchaseOrderNumber]
		FROM [Sales].[Orders]

	END
	ELSE

		INSERT INTO #temp (
			[OrderID],
			[CustomerID],
			[CustomerPONum])
		SELECT
			[OrderID],
			[CustomerID],
			[CustomerPurchaseOrderNumber]
		FROM [Sales].[Orders]
		WHERE [CustomerID] = @CustomerID

	END

	SELECT 
		[t].[OrderID], 
		[t].[CustomerID], 
		SUM([ol].[UnitPrice]*[ol].Quantity) [ItemTotal]
	FROM [Sales].[OrderLines] [ol]
	JOIN #temp [t]
		ON ol.[OrderID] = [t].[OrderID]
	GROUP BY [t].[OrderID], [t].[CustomerID]
	ORDER BY [t].[OrderID]

	DROP TABLE #temp


GO


/*
	Create InMem table type
*/
CREATE TYPE dbo.OrderInfo  
    AS TABLE (
		[RowNum] INT IDENTITY (1,1) NOT NULL,
		[OrderID] INT PRIMARY KEY NONCLUSTERED, 
		[CustomerID] INT,
		[CustomerPONum] NVARCHAR(40) 
	    )  
    WITH  
        (MEMORY_OPTIMIZED = ON);  


/*
	Create SP that uses InMem table type
*/
DROP PROCEDURE IF EXISTS [NiceToTempdb];  
GO

CREATE PROCEDURE [NiceToTempdb] @CustomerID INT = NULL
AS
BEGIN
	SET NOCOUNT ON

	DBCC TRACEON (2453)

	DECLARE @OrderTT dbo.OrderInfo  

	IF @CustomerID IS NULL
	BEGIN
		INSERT INTO @OrderTT (
			[OrderID],
			[CustomerID],
			[CustomerPONum])
		SELECT
			[OrderID],
			[CustomerID],
			[CustomerPurchaseOrderNumber]
		FROM [Sales].[Orders]

	END
	ELSE

		INSERT INTO @OrderTT (
			[OrderID],
			[CustomerID],
			[CustomerPONum])
		SELECT
			[OrderID],
			[CustomerID],
			[CustomerPurchaseOrderNumber]
		FROM [Sales].[Orders]
		WHERE [CustomerID] = @CustomerID

	END

	SELECT 
		[t].[OrderID], 
		[t].[CustomerID], 
		SUM([ol].[UnitPrice]*[ol].Quantity) [ItemTotal]
	FROM [Sales].[OrderLines] [ol]
	JOIN @OrderTT [t]
		ON ol.[OrderID] = [t].[OrderID]
	GROUP BY [t].[OrderID], [t].[CustomerID]
	ORDER BY [t].[OrderID]
	
	DBCC TRACEOFF (2453)
GO


USE [master];
GO
ALTER DATABASE [WideWorldImporters] SET QUERY_STORE (
	OPERATION_MODE = READ_WRITE,
	CLEANUP_POLICY = (STALE_QUERY_THRESHOLD_DAYS = 30),  
	DATA_FLUSH_INTERVAL_SECONDS = 60,
	INTERVAL_LENGTH_MINUTES = 5, 
	MAX_STORAGE_SIZE_MB = 512,
	QUERY_CAPTURE_MODE = ALL,
	SIZE_BASED_CLEANUP_MODE = AUTO,
	MAX_PLANS_PER_QUERY = 200);

GO


/*
	clear QS before start
*/
ALTER DATABASE [WideWorldImporters] SET QUERY_STORE CLEAR;
GO


/*
	Run external scripts
*/
USE [WideWorldImporters];
GO


SELECT
	[qst].[query_text_id],
	[qsq].[query_id], 
	[qsp].[plan_id],
	OBJECT_NAME([qsq].[object_id]),
	[qst].[query_sql_text],
	TRY_CONVERT(XML, [qsp].[query_plan]) [query_plan],
	[rs].[count_executions],
	[rs].[avg_duration],
	[rs].[avg_logical_io_reads],
	[rs].[avg_cpu_time],
	([rs].[count_executions]*[rs].[avg_cpu_time]) [TotalCPU]
FROM [sys].[query_store_query] [qsq] 
JOIN [sys].[query_store_query_text] [qst]
	ON [qsq].[query_text_id] = [qst].[query_text_id]
JOIN [sys].[query_store_plan] [qsp] 
	ON [qsq].[query_id] = [qsp].[query_id]
JOIN [sys].[query_store_runtime_stats] [rs] 
	ON [qsp].[plan_id] = [rs].[plan_id]
WHERE [qsq].[object_id] <> 0
AND [qst].[query_sql_text] LIKE 'SELECT%'
ORDER BY [qst].[query_text_id] DESC;
GO

SELECT
	[qst].[query_text_id],
	[qsq].[query_id], 
	[qsp].[plan_id],
	OBJECT_NAME([qsq].[object_id]),
	[qst].[query_sql_text],
	TRY_CONVERT(XML, [qsp].[query_plan]) [query_plan],
	[rs].[count_executions],
	[rs].[avg_duration],
	[rs].[avg_logical_io_reads],
	[rs].[avg_cpu_time],
	([rs].[count_executions]*[rs].[avg_cpu_time]) [TotalCPU]
FROM [sys].[query_store_query] [qsq] 
JOIN [sys].[query_store_query_text] [qst]
	ON [qsq].[query_text_id] = [qst].[query_text_id]
JOIN [sys].[query_store_plan] [qsp] 
	ON [qsq].[query_id] = [qsp].[query_id]
JOIN [sys].[query_store_runtime_stats] [rs] 
	ON [qsp].[plan_id] = [rs].[plan_id]
WHERE [qsq].[object_id] <> 0
AND [qst].[query_sql_text] LIKE 'INSERT%'
ORDER BY [qst].[query_text_id] DESC;
GO
   
    